// Arduino Nano Router Restarter
// Written by Theron Wierenga, February 2016

// CS = pin 10 (Uno, Nano), pin 52 (Mega)
// MOSI pin 11 (Uno, Nano), pin 51 (Mega) to DIN on DS3234 pin 17
// MISO pin 12 (Uno, Nano), pin 50 (Mega) to DOUT on DS3234 pin 19
// SCK  pin 13 (Uno, Nano), pin 52 (Mega) to SCLK on DS3234 pin 18 or 20

#define AM 0
#define PM 1
#define CHAN1 2
#define CHAN2 3
// SPI Library is used to read the DS3234 RTC
#include <SPI.h>
// Wire Library is used for I2C communication with the LCD display
#include <Wire.h>
// The LCD library
#include <LiquidCrystal_I2C.h>
// Create and LCD object
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address for 20x4 LCD

const int cs = 10; //chip select for DS3234
int count = 0;
byte second = 0, minute = 1, hour = 2, ampm, dayOfWeek, dayOfMonth, month, year, f1224;
byte shour1 = 3, sminute1 = 0, sampm1 = 0, shour2 = 3, sminute2 = 1, sampm2 = 0; // start times
byte ehour1 = 3, eminute1 = 0, eampm1 = 0, ehour2 = 3, eminute2 = 1, eampm2 = 0; // end times
String months[12] = {"January","February","March","April","May","June","July","August","September","October","November","December"};
String daysOfWeek[7] = {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"};
// Create two 20 charactre strings
String sDate = "12345678901234567890";
String sDayofWeek = "12345678901234567890";

void setup()
{ 
  // Initialize the DS3234 RTC
  RTC_init_DS3234();
  //day(1-31), month(1-12), year(0-99), dayofweek (1-7), ampm (0-1), hour(0-23), minute(0-59), second(0-59), 12/24 hr (0-1)
  //                                    Sun = 1          0 = am, 1 = pm                                 0 = 24 hr., 1 = 12 hr.
     f1224 = 1;
  // Set the Time and Date then comment out this line and compile and upload the program to the Arduino again
  // By doing this if the Arduino loses power the RTC will not be reprogrammed with the earlier time and date  
  // SetTimeDate_DS3234(24,2,16,4,1,2,2,30,f1224); 

  // Now read the time and date to display on the LCD
  ReadTimeDate_DS3234(&second, &minute, &hour, &ampm, &dayOfWeek, &dayOfMonth, &month, &year);
  if (ampm > 0) {ampm = 1;}   // PM returned as a 32 at bit 5
  lcd.begin(20, 4);
  lcd.setCursor(0, 0);
  dateString();
  Display_Date();
  // Set our times on the two channels
  setStartTime(CHAN1, 3, 0, AM);
  setEndTime(CHAN1, 3, 1, AM);
  setStartTime(CHAN2, 3, 2, AM);
  setEndTime(CHAN2, 3, 3, AM);
}

void loop()
{
 // Read the Time and Date 
 // *******************************************************************************************
 if (count == 0)
 {
    ReadTimeDate_DS3234(&second, &minute, &hour, &ampm, &dayOfWeek, &dayOfMonth, &month, &year);
    if (ampm > 0) {ampm = 1;}   // PM returned as a 32 at bit 5
    // Now display the results
    Display_Time();
    // This would need to be changed if 24 hour clock
    if ((hour == 12) && (minute == 0) && (second == 0) && ( count == 0))
    {
      // We do this because we only need to change the date once a day, 12 hour clock will do it twice
      dateString();
      Display_Date();
    }

    // Now see if we need to set or reset a relay
    checkTime();

 }
 // This is done because we do not need to read the time hundreds of times a second
 count++;
 if (count == 15) {count = 0;}
}

// *************************************************************************************
// Convert normal decimal (binary) numbers to binary coded decimal
byte decToBcd(byte val)
{
  return ( (val/10*16) + (val%10) );
}

// *************************************************************************************
// Convert binary coded decimal to normal decimal (binary) numbers
byte bcdToDec(byte val)
{
  return ( (val/16*10) + (val%16) );
}

// ************************************************************************************
// Initialize DS3234
void RTC_init_DS3234(){ 
	  pinMode(cs,OUTPUT);         // chip select
	  SPI.begin();                // start the SPI library:
	  SPI.setBitOrder(MSBFIRST);  // set bit order
	  SPI.setDataMode(SPI_MODE1); // both mode 1 & 3 work
	  digitalWrite(cs, LOW);  
	  SPI.transfer(0x8E);         //set control register 
	  SPI.transfer(0x60);         //0x60 = disable osciallator and battery, SQ wave @1hz, temp compensation, alarms disabled
	  digitalWrite(cs, HIGH);
	  delay(10);
}
//======================================================================================
// Set Time and Date info
void SetTimeDate_DS3234(byte d, byte mo, byte y, byte dw, byte ampm, byte h, byte mi, byte s, byte f1224)
{ 
          digitalWrite(cs, LOW);
          SPI.transfer(0x80);                              // starting address
          SPI.transfer(decToBcd(s));                       // send seconds
          SPI.transfer(decToBcd(mi));                      // send minutes
          byte temp = 0;
          if (f1224 == 1)                                  // It's 12 hr.
          {
              temp = temp | B01000000;                     // This sets 12 hr.
              if (ampm == 1) {temp = temp | B00100000;}    // This sets pm in am/pm
              SPI.transfer(decToBcd(h) | temp);            // send hours for 12 hours
          }
          else
          {
              SPI.transfer(decToBcd(h));                   // send hours for 24 hours
          }
          SPI.transfer(decToBcd(dw));                      // send day of week
          SPI.transfer(decToBcd(d));                       // send day of month
          SPI.transfer(decToBcd(mo));                      // send month
          SPI.transfer(decToBcd(y));                       // send year
          digitalWrite(cs, HIGH);
}

//=======================================================================================
// Read Time and Date info
void ReadTimeDate_DS3234(byte *second, byte *minute, byte *hour, byte *ampm, byte *dayOfWeek, byte *dayOfMonth, byte *month, byte *year)
{
          byte temphour = 0;
          digitalWrite(cs, LOW);
          SPI.transfer(0x00);                               // we will be reading
	  *second = bcdToDec(SPI.transfer(0x00) & 0x7f);    // get seconds
          *minute = bcdToDec(SPI.transfer(0x00));           // get minutes
          temphour = SPI.transfer(0x00);
          if (f1224 == 1)
          {
             *hour = bcdToDec(temphour & 0x1f);             // get 12 hour
          }
          else
          {
             *hour = bcdToDec(temphour & 0x3f);             // get 24 hour
          }
          *ampm = (temphour & B00100000) >> 5;              // get am/pm
          *dayOfWeek = bcdToDec(SPI.transfer(0x00));        // get day of week
          *dayOfMonth = bcdToDec(SPI.transfer(0x00));       // get day of month
          *month = bcdToDec(SPI.transfer(0x00));            // get month
          *year = bcdToDec(SPI.transfer(0x00));             // get year
          digitalWrite(cs, HIGH);
}

// ************************************************************************************
// Display Time
void Display_Time()
{
  // For cursor first number is char in line, second number is line
  lcd.setCursor(0, 0);
  if (hour < 10) lcd.print("0");  // Print 0 in front of 0-9 numbers
  lcd.print(hour);
  lcd.print(":");
  if (minute < 10) lcd.print("0");
  lcd.print(minute);
  lcd.print(":");
  if (second < 10) lcd.print("0");
  lcd.print(second);
  lcd.print(" ");
  if (ampm == 0)
  {
     lcd.print("AM"); 
  }
  else
  {
    lcd.print("PM"); 
  }
  
}

// ************************************************************************************
// Display Date
void Display_Date()
{
    lcd.setCursor(0, 3);
    lcd.print(sDate);
    lcd.setCursor(0, 2);
    lcd.print(sDayofWeek);
    
}

// ************************************************************************************
// Build Date and Day of Week Strings
void dateString()
{
    sDate = months[month - 1];
    sDate = sDate + " ";
    sDate = sDate + String(dayOfMonth);
    sDate = sDate + ", 20";
    sDate = sDate + String(year);
    while (sDate.length() < 20)
    {
        sDate = sDate + " ";
    }
    sDayofWeek = daysOfWeek[dayOfWeek - 1];
    while (sDayofWeek.length() < 20)
    {
        sDayofWeek = sDayofWeek + " ";
    }
}

// ***********************************************************************************
// Set Start Time
// Channel number 1-2, hour, minute, AM or PM
void setStartTime(byte chan, byte h, byte m, byte apm)
{
    if(chan == CHAN1)
    {
        shour1 = h;
        sminute1 = m;
        sampm1 = apm;
    }
    if(chan == CHAN2)
    {
        shour2 = h;
        sminute2 = m;
        sampm2 = apm;
    }
}

// ***********************************************************************************
// Set End Time
void setEndTime(byte chan, byte h, byte m, byte apm)
{
    if(chan == CHAN1)
    {
        ehour1 = h;
        eminute1 = m;
        eampm1 = apm;
    }
    if (chan == CHAN2)
    {
        ehour2 = h;
        eminute2 = m;
        eampm2 = apm;
    }
}

// **********************************************************************************
// check Time
void checkTime()
{
    if ((hour == shour1) && (minute == sminute1) && (ampm == sampm1))
    {
        // Activate channel 1
        digitalWrite(CHAN1, HIGH);
        lcd.setCursor(0, 1);
        lcd.print("Channel 1 Activate  ");
    }
    if ((hour == shour2) && (minute == sminute2) && (ampm == sampm2))
    {
        // Activate channel 2
         digitalWrite(CHAN2, HIGH);
         lcd.setCursor(0, 1);
         lcd.print("Channel 2 Activate  ");
    }

    if ((hour == ehour1) && (minute == eminute1) && (ampm == eampm1))
    {
        // Deactivate channel 1
        digitalWrite(CHAN1, LOW);
        lcd.setCursor(0, 1);
        lcd.print("Channel 1 Deactivate");
    }
    if ((hour == ehour2) && (minute == eminute2) && (ampm == eampm2))
    {
        // Deactivate channel 2
        digitalWrite(CHAN2, LOW);
        lcd.setCursor(0, 1);
        lcd.print("Channel 2 Deactivate");
    }    
}

